<sidebar>
  <ul id="sidebar" class="sidebar-nav">
    <li each={store.navs} class="top-level" >
      <span>{name}</span>
      <ul class="sub-nav">
        <li each={children} if={children}>
          <a href ="#{path}" onclick="{() => switchNav(path)}" class="{active: store.nav.path == path}">{name}</a>
        </li>
      </ul>
    </li>
  </ul>
  <script>
  var self = this;
  self.switchNav = function(path) {
    store.nav.path = path;
  }
  store.nav.path = getHash();
  store.on('apiLoaded', function() {
    self.update();
  });
  </script>
</sidebar>

<app-router>
  <div ref="view" data-is={component}></div>
  <script>
  var self = this
  route('/*', function(name) {
    console.log(name);
    self.component = name;
    self.update();
  });
  route('/api/*/*', function(api, path) {
    store.api.spec = store.apis[api];
    store.trigger('apiChanged');
    self.component = 'apidoc';
    self.update();
  });
  route.start();
  route.exec();
  </script>
</app-router>

<model-prop>
  <table class="table">
    <tr each={def, name in spec.properties}>
      <td>{name}</td>
      <td>{def.type}</td>
      <td>{def.description}</td>
      <td if = {def.type == "array" && def.items.$ref}>
      </td>
      <td if = {def.type == "object" && def.items.$ref}>
      </td>
    </tr>
  </ul>
  <script>
  this.spec = opts.spec
  </script>
</model-prop>

<api-query-params>
  <div class="card-block">
    <h4 class="card-title">Url参数(QueryString)</h4>
    <table class="api-parameter table table-bordered">
      <thead>
        <tr>
          <th>参数名</th>
          <th>参数类型</th>
          <th>说明</th>
        </tr>
      </thead>
      <tbody>
        <tr>
          <td>timestamp</td>
          <td>long</td>
          <td>时间戳，即当前时间距离1970年1月1日的毫秒数</td>
        </tr>
        <tr>
          <td>clientId</td>
          <td>string</td>
          <td>平台分配的clientId</td>
        </tr>
        <tr>
          <td>signature</td>
          <td>签名</td>
          <td>参考<a href="/doc#/sign">签名算法</a></td>
        </tr>
      </tbody>
    </table>
  </table>
  </div>
</api-query-params>

<api-body-params>
  <div class="card-block">
    <h4 class="card-title">参数列表</h4>
    <h6 class="card-subtitle"><span class="badge badge-api">application/json</span></h6>
    <table class="api-parameter table table-bordered">
      <thead>
        <tr>
          <th>参数名</th>
          <th>参数类型</th>
          <th>说明</th>
        </tr>
      </thead>
      <tbody>
        <tr each={opts.params}>
          <td>{name}<span if={required} class="badge badge-warning required">必填</span></td>
          <td>{type}</td>
          <td>{description}</td>
        </tr>
      </tbody>
    </table>
  </div>
</api-body-params>

<prop-table>
  <table>
    <thead>
      <tr>
        <td>字段名</td>
        <td>类型</td>
        <td>说明</td>
        <td></td>
      </tr>
    </thead>
    <tbody>
      <tr each={prop, name in opts.props}>
        <td>{name}<em if={prop.required}></em></td>
        <td>{prop.type}</td>
        <td>{prop.description}</td>
        <td>
          <response-schema if={prop.type == "object" && prop.schema.$ref} spec={schemaDefOf(prop.schema.$ref)}></response-schema>
          <span if={prop.type == "array" }>
            array(<response-schema if={prop.type == "array" && prop.items.schema.$ref} spec={schemaDefOf(prop.items.schema.$ref)}></response-schema>)
          </span>

        </td>
      </tr>
    </tbody>
  </table>
</prop-table>

<response-schema>
  <span class="split">&nbsp;&nbsp;</span>
  <div if={opts.spec.type == "array"}>
    <span onclick={toggleShow}>
      <span class="badge badge-primary"><i class={fa: true, "fa-angle-right": !showNested, "fa-angle-down": showNested } aria-hidden="true"></i>({opts.spec.type})</span>
    </span>
    <response-schema spec={opts.spec.items} if={showNested}></response-schema>
    <span if={opts.spec.items.type != "object"}>{opts.spec.items.type}</span>
  </div>
  <div if={opts.spec.type == "object"}>
    <a onclick={toggleShow}>
      <span class="badge badge-primary"><i class={fa: true, "fa-angle-right": !showNested, "fa-angle-down": showNested } aria-hidden="true"></i>({opts.spec.type})</span>
    </a>
    <prop-table if={showNested && opts.spec.schema && opts.spec.schema.$ref} props={schemaDefOf(opts.spec.schema.$ref).properties}></prop-table>
    <prop-table if={showNested && opts.spec.properties} props={opts.spec.properties}></prop-table>
  </div>
  <script>
  this.showNested = false;
  toggleShow(e) {
    this.showNested = !this.showNested;
  }
  </script>
</response-schema>
<api-response>
  <div class="api-response card-block">
    <h4 class="card-title">返回结果(json)</h4>
    <table class="api-parameter table table-bordered">
      <thead>
        <tr>
          <th>字段名</th>
          <th>字段类型</th>
          <th>必须存在</th>
          <th>说明</th>
        </tr>
      </thead>
      <tbody>
        <tr>
          <td>sucess</td>
          <td>boolean</td>
          <td>否</td>
          <td>成功为 true, 失败为 false </td>
        </tr>
        <tr>
          <td>errcode</td>
          <td>int</td>
          <td>否</td>
          <td>失败时的错误码</td>
        </tr>
        <tr>
          <td>errmsg</td>
          <td>string</td>
          <td>否</td>
          <td>失败时的错误信息</td>
        </tr>
        <tr>
          <td>result</td>
          <td>{opts.response.type}</td>
          <td>否</td>
          <td>
            {opts.response.description}
            <response-schema if={opts.response.type == "object" || opts.response.type == "array"} spec={opts.response}></response-schema>
          </td>
        </tr>
      </tbody>
    </table>
  </div>
</api-response>
<api-example>
  <div class="card-block example" data-path="{opts.info.path}">
    <h4 class="card-title">示例</h4>
    <div>
      <h6>请求</h6>
      <pre class="req-code-pre"><code></code></pre>
      <h6>结果</h6>
      <pre class="res-code-pre"><code></code></pre>
    </div>
  </div>
  <script>
  var self = this;
  self.rehightlight = function() {
    self.updateExample();
    var root =$('div[data-path='+ $.escapeSelector(opts.info.path) + ']')
    $(root).find('pre.req-code-pre').replaceWith('<pre class="req-code-pre"><code>' + self.requestExample + '</code></pre>');
    $(root).find('pre.res-code-pre').replaceWith('<pre class="res-code-pre"><code>' + self.responseExample + '</code></pre>');
    $(root).find('pre code').each(function(i, e) {
      hljs.highlightBlock(e);
    });


  }
  self.updateExample = function() {
    self.requestExample =
      `$ curl -XPOST https://${store.api.spec.host}${store.api.spec.basePath}${opts.info.path}?timestamp=xxx&clientId=yyy&signature=zzz -d '` +
      getRequestExample(opts.spec.parameters) + "'";
    self.responseExample = getExampleResponse(responseOf(opts.info.path))
  };

  this.on('updated', function(e) {
    self.rehightlight();
  });
  self.on('mount',function(e) {
    self.rehightlight();
  });

  </script>
</api-example>

<doc-item>
  <div class="apidoc-item">
    <div class="card-block">
      <h3 class="card-title">{opts.spec.description}</h3>
      <h6 class="card-subtitle">
        <span class="badge badge-api">{opts.info.method}</span>
        <span class="badge badge-api">{store.api.spec.basePath}{opts.info.path}</span>
      </h6>
    </div>
    <api-query-params></api-query-params>
    <api-body-params params={opts.spec.parameters}></api-body-params>
    <api-response response={opts.spec.responses['200']}></api-response>
    <api-example spec={opts.spec} info={opts.info}></api-example>
  </div>
  <script>
  var self = this;
  schemaDefOf(ref) {
    var re = /#\/([^\/]+)\/([^\/]+)/;
    var gs = re.exec(ref);
    return store.api.spec[gs[1]][gs[2]];
  }
  this.on('mount', function() {
    hljs.initHighlighting();
  });
  opts.spec;
  </script>
</doc-item>

<apidoc>
  <div class="apidoc" if={store.api.spec}>
    <div class="doc-list">
      <div each={ methods, path in store.api.spec.paths} if={isPathMatch(path)}>
        <doc-item each={ s, method in methods } info={{"path":parent.path, "method": method}} spec={s}>
        </doc-item>
      </div>
    </div>
  </div>
  <script>
  var self = this;
  self.spec = store.api.spec;
  store.on('apiChanged', function() {
    self.spec = store.api.spec;
  });

  self.isPathMatch = function(subPath) {
    return (self.spec.basePath + subPath) == store.nav.path;
  }
  </script>
</apidoc>
